@@ -31,18 +31,9 @@
 */
 
 #include <stddef.h>
+#include <stdbool.h>
 #include <string.h>  // memcpy
 
-#if defined(_MSC_VER) && (_MSC_VER <= 1700)
-  /* for vs2012/11.0/1700 and earlier Visual Studio compilers */
-# define bool   int
-# define false  0
-# define true   1
-#else
-# include <stdbool.h>
-#endif
-
-
 #ifdef _WIN32
 #include "winconfig.h"
 #else
